@@ -16,13 +16,23 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import re
 
-from mercurial import pycompat
+from typing import (
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Pattern,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 from .qtcore import (
+    QModelIndex,
     QSettings,
     Qt,
 )
@@ -32,6 +42,7 @@ from .qtgui import (
     QFormLayout,
     QFrame,
     QHBoxLayout,
+    QIcon,
     QLabel,
     QLineEdit,
     QListWidget,
@@ -50,22 +61,25 @@ from . import qtlib
 if hglib.TYPE_CHECKING:
     from typing import (
         Any,
-        Optional,
     )
 
     from ..util.typelib import (
         IniConfig,
     )
 
-DEFAULTICONNAME = 'tools-spanner-hammer'
+_W = TypeVar('_W', bound=QWidget)
+
+DEFAULTICONNAME: str = 'tools-spanner-hammer'
 
 
 class ToolsFrame(QFrame):
-    def __init__(self, ini, parent=None, **opts):
-        # type: (IniConfig, Optional[QWidget], Any) -> None
+    def __init__(
+        self, ini: "IniConfig", parent: Optional[QWidget]=None, **opts
+    ) -> None:
         QFrame.__init__(self, parent, **opts)
         self.widgets = []
         self.ini = ini
+        self.tortoisehgtools: Dict[str, Dict[str, Union[str, bool]]]
         self.tortoisehgtools, guidef = hglib.tortoisehgtools(self.ini)
         self.setValue(self.tortoisehgtools)
 
@@ -83,7 +97,7 @@ class ToolsFrame(QFrame):
         self.locationcombo = QComboBox(self,
             toolTip=_('Select the toolbar or menu to change'))
 
-        def selectlocation(index):
+        def selectlocation(index: int) -> None:
             location = self.locationcombo.itemData(index)
             for widget in self.widgets:
                 if widget.location == location:
@@ -149,7 +163,7 @@ class ToolsFrame(QFrame):
         # Ensure that the first location list is shown
         selectlocation(0)
 
-    def getCurrentToolList(self):
+    def getCurrentToolList(self) -> Optional["ToolListBox"]:
         index = self.locationcombo.currentIndex()
         location = self.locationcombo.itemData(index)
         for widget in self.widgets:
@@ -157,7 +171,7 @@ class ToolsFrame(QFrame):
                 return widget
         return None
 
-    def addToList(self):
+    def addToList(self) -> None:
         gtl = self.globaltoollist
         row = gtl.currentIndex().row()
         if row < 0:
@@ -168,22 +182,22 @@ class ToolsFrame(QFrame):
         toolname = item.text()
         self.forwardToCurrentToolList('addOrInsertItem', toolname, icon=item.icon())
 
-    def forwardToCurrentToolList(self, funcname, *args, **opts):
+    def forwardToCurrentToolList(self, funcname: str, *args, **opts) -> None:
         w = self.getCurrentToolList()
         if w is not None:
             getattr(w, funcname)(*args, **opts)
         return None
 
-    def newTool(self):
+    def newTool(self) -> None:
         td = CustomToolConfigDialog(self)
-        res = td.exec_()
+        res = td.exec()
         if res:
             toolname, toolconfig = td.value()
             self.globaltoollist.addOrInsertItem(
                 toolname, icon=toolconfig.get('icon', None))
             self.tortoisehgtools[toolname] = toolconfig
 
-    def editTool(self, row=None):
+    def editTool(self, row: Optional[int] = None) -> None:
         gtl = self.globaltoollist
         if row is None:
             row = gtl.currentIndex().row()
@@ -194,8 +208,8 @@ class ToolsFrame(QFrame):
             toolname = item.text()
             td = CustomToolConfigDialog(
                 self, toolname=toolname,
-                toolconfig=self.tortoisehgtools[str(toolname)])
-            res = td.exec_()
+                toolconfig=self.tortoisehgtools[toolname])
+            res = td.exec()
             if res:
                 toolname, toolconfig = td.value()
                 icon = toolconfig.get('icon', '')
@@ -207,11 +221,12 @@ class ToolsFrame(QFrame):
                 gtl.setCurrentRow(row)
                 self.tortoisehgtools[toolname] = toolconfig
 
-    def editToolItem(self, item):
+    def editToolItem(self, item: QModelIndex) -> None:
         self.editTool(item.row())
 
-    def editToolFromName(self, name):
+    def editToolFromName(self, idx: QModelIndex) -> None:
         # [TODO] connect to toollist doubleClick (not global)
+        name = idx.data(Qt.ItemDataRole.DisplayRole)
         gtl = self.globaltoollist
         if name == gtl.SEPARATOR:
             return
@@ -221,27 +236,29 @@ class ToolsFrame(QFrame):
                 self.editTool(row)
                 return
 
-    def deleteCurrentTool(self):
+    def deleteCurrentTool(self) -> None:
         row = self.globaltoollist.currentIndex().row()
         if row >= 0:
             item = self.globaltoollist.item(row)
-            itemtext = str(item.text())
+            itemtext = item.text()
             self.globaltoollist.deleteTool(row=row)
 
             self.deleteTool(itemtext)
             self.forwardToCurrentToolList('removeInvalid', self.value())
 
-    def deleteTool(self, name):
+    def deleteTool(self, name: str) -> None:
         try:
             del self.tortoisehgtools[name]
         except KeyError:
             pass
 
-    def applyChanges(self, ini):
+    def applyChanges(self, ini: "IniConfig") -> bool:
         # widget.value() returns the _NEW_ values
         # widget.curvalue returns the _ORIGINAL_ values (yes, this is a bit
         # misleading! "cur" means "current" as in currently valid)
-        def updateIniValue(section, key, newvalue):
+        def updateIniValue(
+            section: str, key: str, newvalue: Optional[bytes]
+        ) -> None:
             section = hglib.fromunicode(section)
             key = hglib.fromunicode(key)
             try:
@@ -266,11 +283,7 @@ class ToolsFrame(QFrame):
                       'icon', 'location', 'enable', 'showoutput',)
         for name in self.curvalue:
             for field in fieldnames:
-                try:
-                    keyname = '%s.%s' % (name, field)
-                    del ini[section][keyname]
-                except KeyError:
-                    pass
+                updateIniValue(section, '%s.%s' % (name, field), None)
 
         tools = self.value()
         for name in tools:
@@ -282,7 +295,7 @@ class ToolsFrame(QFrame):
                 # value may be bool if originating from hglib.tortoisehgtools()
                 if value != '':
                     updateIniValue(section, keyname,
-                                   hglib.fromunicode(pycompat.unicode(value)))
+                                   hglib.fromunicode(str(value)))
 
         # 2. Save the new guidefs
         for n, toollistwidget in enumerate(self.widgets):
@@ -298,13 +311,13 @@ class ToolsFrame(QFrame):
         return emitChanged
 
     ## common APIs for all edit widgets
-    def setValue(self, curvalue):
+    def setValue(self, curvalue: Dict[str, Dict[str, Union[str, bool]]]) -> None:
         self.curvalue = dict(curvalue)
 
-    def value(self):
+    def value(self) -> Dict[str, Dict[str, Union[str, bool]]]:
         return self.tortoisehgtools
 
-    def isDirty(self):
+    def isDirty(self) -> bool:
         for toollistwidget in self.widgets:
             if toollistwidget.isDirty():
                 return True
@@ -312,7 +325,7 @@ class ToolsFrame(QFrame):
             return True
         return self.tortoisehgtools != self.curvalue
 
-    def refresh(self):
+    def refresh(self) -> None:
         self.tortoisehgtools, guidef = hglib.tortoisehgtools(self.ini)
         self.setValue(self.tortoisehgtools)
         self.globaltoollist.refresh()
@@ -321,8 +334,9 @@ class ToolsFrame(QFrame):
 
 
 class HooksFrame(QFrame):
-    def __init__(self, ini, parent=None, **opts):
-        # type: (IniConfig, Optional[QWidget], Any) -> None
+    def __init__(
+        self, ini: "IniConfig", parent: Optional[QWidget]=None, **opts
+    ) -> None:
         super(HooksFrame, self).__init__(parent, **opts)
         self.ini = ini
         # The frame is created empty, and will be populated on 'refresh',
@@ -351,11 +365,15 @@ class HooksFrame(QFrame):
         buttonbox.addStretch()
         topbox.addLayout(buttonbox)
 
-    def newHook(self):
+    def newHook(self) -> None:
         td = HookConfigDialog(self)
-        res = td.exec_()
+        res = td.exec()
         if res:
-            hooktype, command, hookname = td.value()
+            uhooktype, ucommand, uhookname = td.value()
+            hooktype = hglib.fromunicode(uhooktype)
+            command = hglib.fromunicode(ucommand)
+            hookname = hglib.fromunicode(uhookname)
+
             # Does the new hook already exist?
             hooks = self.value()
             if hooktype in hooks:
@@ -368,7 +386,7 @@ class HooksFrame(QFrame):
                             _('Replace existing hook?'),
                             _('There is an existing %s.%s hook.\n\n'
                             'Do you want to replace it?')
-                            % (hooktype, hookname),
+                            % (uhooktype, uhookname),
                             parent=self):
                         return
                     # Delete existing matching hooks in reverse order
@@ -380,7 +398,7 @@ class HooksFrame(QFrame):
             self.hooktable.setSortingEnabled(False)
             row = self.hooktable.rowCount()
             self.hooktable.insertRow(row)
-            for c, text in enumerate((hooktype, hookname, command)):
+            for c, text in enumerate((uhooktype, uhookname, ucommand)):
                 self.hooktable.setItem(row, c, QTableWidgetItem(text))
             # Make the hook column not editable (a dialog is used to edit it)
             itemhook = self.hooktable.item(row, 0)
@@ -389,7 +407,7 @@ class HooksFrame(QFrame):
             self.hooktable.resizeColumnsToContents()
             self.updatebuttons()
 
-    def editHook(self, r, c=0):
+    def editHook(self, r: int, c: int = 0) -> bool:
         if r < 0:
             r = 0
         numrows = self.hooktable.rowCount()
@@ -404,7 +422,7 @@ class HooksFrame(QFrame):
         command = self.hooktable.item(r, 2).text()
         td = HookConfigDialog(self, hooktype=hooktype,
                               command=command, hookname=hookname)
-        res = td.exec_()
+        res = td.exec()
         if res:
             hooktype, command, hookname = td.value()
             # Update the table
@@ -420,10 +438,10 @@ class HooksFrame(QFrame):
             self.hooktable.resizeColumnsToContents()
         return bool(res)
 
-    def editCurrentHook(self):
+    def editCurrentHook(self) -> None:
         self.editHook(self.hooktable.currentRow())
 
-    def deleteHook(self, row=None):
+    def deleteHook(self, row: Optional[int] = None) -> None:
         if row is None:
             row = self.hooktable.currentRow()
             if row < 0:
@@ -432,10 +450,15 @@ class HooksFrame(QFrame):
         self.hooktable.resizeColumnsToContents()
         self.updatebuttons()
 
-    def deleteCurrentHook(self):
+    def deleteCurrentHook(self) -> None:
         self.deleteHook()
 
-    def findHooks(self, hooktype=None, hookname=None, command=None):
+    def findHooks(
+        self,
+        hooktype: Optional[bytes] = None,
+        hookname: Optional[bytes] = None,
+        command: Optional[bytes] = None,
+    ) -> List[int]:
         matchingrows = []
         for r in range(self.hooktable.rowCount()):
             currhooktype = hglib.fromunicode(self.hooktable.item(r, 0).text())
@@ -448,12 +471,12 @@ class HooksFrame(QFrame):
                 matchingrows.append(r)
         return matchingrows
 
-    def updatebuttons(self):
+    def updatebuttons(self) -> None:
         tablehasitems = self.hooktable.rowCount() > 0
         self.btnedit.setEnabled(tablehasitems)
         self.btndelete.setEnabled(tablehasitems)
 
-    def applyChanges(self, ini):
+    def applyChanges(self, ini: "IniConfig") -> bool:
         # widget.value() returns the _NEW_ values
         # widget.curvalue returns the _ORIGINAL_ values (yes, this is a bit
         # misleading! "cur" means "current" as in currently valid)
@@ -463,12 +486,12 @@ class HooksFrame(QFrame):
         emitChanged = True
 
         # 1. Delete the previous hook configurations
-        section = 'hooks'
+        section = b'hooks'
         hooks = self.curvalue
         for hooktype in hooks:
             for keyname in hooks[hooktype]:
                 if keyname:
-                    keyname = '%s.%s' % (hooktype, keyname)
+                    keyname = b'%s.%s' % (hooktype, keyname)
                 else:
                     keyname = hooktype
                 try:
@@ -480,7 +503,7 @@ class HooksFrame(QFrame):
         for hooktype in hooks:
             for field in sorted(hooks[hooktype]):
                 if field:
-                    keyname = '%s.%s' % (hooktype, field)
+                    keyname = b'%s.%s' % (hooktype, field)
                 else:
                     keyname = hooktype
                 value = hooks[hooktype][field]
@@ -489,10 +512,10 @@ class HooksFrame(QFrame):
         return emitChanged
 
     ## common APIs for all edit widgets
-    def setValue(self, curvalue):
+    def setValue(self, curvalue: Dict[bytes, Dict[bytes, bytes]]) -> None:
         self.curvalue = dict(curvalue)
 
-    def value(self):
+    def value(self) -> Dict[bytes, Dict[bytes, bytes]]:
         hooks = {}
         for r in range(self.hooktable.rowCount()):
             hooktype = hglib.fromunicode(self.hooktable.item(r, 0).text())
@@ -503,13 +526,13 @@ class HooksFrame(QFrame):
             hooks[hooktype][hookname] = command
         return hooks
 
-    def isDirty(self):
+    def isDirty(self) -> bool:
         return self.value() != self.curvalue
 
-    def gethooks(self):
+    def gethooks(self) -> Dict[bytes, Dict[bytes, bytes]]:
         hooks = {}
         for key, value in self.ini.items(b'hooks'):
-            keyparts = key.split(b'.', 1)
+            keyparts: List[bytes] = key.split(b'.', 1)
             hooktype = keyparts[0]
             if len(keyparts) == 1:
                 name = b''
@@ -520,7 +543,7 @@ class HooksFrame(QFrame):
             hooks[hooktype][name] = value
         return hooks
 
-    def refresh(self):
+    def refresh(self) -> None:
         hooks = self.gethooks()
         self.setValue(hooks)
         self.hooktable.setSortingEnabled(False)
@@ -544,15 +567,20 @@ class HooksFrame(QFrame):
 
 
 class ToolListBox(QListWidget):
-    SEPARATOR = '------'
-    def __init__(self, ini, parent=None, location=None, minimumwidth=None,
-                 **opts):
-        # type: (IniConfig, Optional[QWidget], Optional[str], Optional[int], Any) -> None
+    SEPARATOR: str = '------'
+    def __init__(
+        self,
+        ini: "IniConfig",
+        parent: Optional[QWidget] = None,
+        location: Optional[str] = None,
+        minimumwidth: Optional[int] = None,
+        **opts
+    ) -> None:
         QListWidget.__init__(self, parent, **opts)
         self.opts = opts
         self.curvalue = None
         self.ini = ini
-        self.location = location
+        self.location: Optional[str] = location
 
         if minimumwidth:
             self.setMinimumWidth(minimumwidth)
@@ -564,7 +592,7 @@ class ToolListBox(QListWidget):
         self.setDragDropMode(QListWidget.DragDropMode.InternalMove)
         self.setDefaultDropAction(Qt.DropAction.MoveAction)
 
-    def _guidef2toollist(self, guidef):
+    def _guidef2toollist(self, guidef: Iterable[str]) -> List[str]:
         toollist = []
         for name in guidef:
             if name == '|':
@@ -575,7 +603,7 @@ class ToolListBox(QListWidget):
             toollist.append(name)
         return toollist
 
-    def _toollist2guidef(self, toollist):
+    def _toollist2guidef(self, toollist: List[str]) -> List[str]:
         guidef = []
         for uname in toollist:
             if uname == self.SEPARATOR:
@@ -588,7 +616,9 @@ class ToolListBox(QListWidget):
             guidef.append(name)
         return guidef
 
-    def addOrInsertItem(self, text, icon=None):
+    def addOrInsertItem(
+        self, text: str, icon: Optional[Union[str, QIcon]] = None
+    ) -> None:
         if text == self.SEPARATOR:
             item = text
         else:
@@ -605,42 +635,46 @@ class ToolListBox(QListWidget):
             self.insertItem(row+1, item)
             self.setCurrentRow(row+1)
 
-    def deleteTool(self, row=None, remove=False):
+    def deleteTool(
+        self, row: Optional[int] = None, remove: bool = False
+    ) -> None:
         if row is None:
             row = self.currentIndex().row()
         if row >= 0:
             self.takeItem(row)
 
-    def addSeparator(self):
+    def addSeparator(self) -> None:
         self.addOrInsertItem(self.SEPARATOR, icon=None)
 
-    def values(self):
+    def values(self) -> List[str]:
         out = []
         for row in range(self.count()):
             out.append(self.item(row).text())
         return out
 
     ## common APIs for all edit widgets
-    def setValue(self, curvalue):
+    def setValue(self, curvalue: List[str]) -> None:
         self.curvalue = curvalue
 
-    def value(self):
+    def value(self) -> List[str]:
         return self._toollist2guidef(list(self.values()))
 
-    def isDirty(self):
+    def isDirty(self) -> bool:
         return self.value() != self.curvalue
 
-    def refresh(self):
+    def refresh(self) -> None:
         toolsdefs, guidef = hglib.tortoisehgtools(self.ini,
             selectedlocation=self.location)
-        self.toollist = self._guidef2toollist(guidef)
+        self.toollist: List[str] = self._guidef2toollist(guidef)
         self.setValue(guidef)
         self.clear()
         for toolname in self.toollist:
             icon = toolsdefs.get(toolname, {}).get('icon', None)
             self.addOrInsertItem(toolname, icon=icon)
 
-    def removeInvalid(self, validtools):
+    def removeInvalid(
+        self, validtools: Dict[str, Dict[str, Union[str, bool]]]
+    ) -> None:
         validguidef = []
         for toolname in self.value():
             if toolname[0] not in '|-':
@@ -657,7 +691,12 @@ class ToolListBox(QListWidget):
 class CustomConfigDialog(QDialog):
     '''Custom Config Dialog base class'''
 
-    def __init__(self, parent=None, dialogname='', **kwargs):
+    def __init__(
+        self,
+        parent: Optional[QWidget] = None,
+        dialogname: str = '',
+        **kwargs
+    ) -> None:
         QDialog.__init__(self, parent, **kwargs)
         self.dialogname = dialogname
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowType.WindowContextHelpButtonHint)
@@ -679,10 +718,15 @@ class CustomConfigDialog(QDialog):
         self.setMaximumHeight(self.sizeHint().height())
         self._readsettings()
 
-    def value(self):
+    def value(self) -> None:
         return None
 
-    def _genCombo(self, items, selecteditem=None, tooltips=None):
+    def _genCombo(
+        self,
+        items: Iterable[str],
+        selecteditem: Optional[str] = None,
+        tooltips: Optional[Iterable[str]] = None,
+    ) -> QComboBox:
         index = 0
         if selecteditem:
             try:
@@ -698,35 +742,41 @@ class CustomConfigDialog(QDialog):
                 combo.setItemData(idx, tooltip, Qt.ItemDataRole.ToolTipRole)
         return combo
 
-    def _addConfigItem(self, parent, label, configwidget, tooltip=None):
+    def _addConfigItem(
+        self,
+        parent: QFormLayout,
+        label: str,
+        configwidget: _W,
+        tooltip: Optional[str]=None,
+    ) -> _W:
         if tooltip:
             configwidget.setToolTip(tooltip)
         parent.addRow(label, configwidget)
         return configwidget
 
-    def okClicked(self):
+    def okClicked(self) -> None:
         errormsg = self.validateForm()
         if errormsg:
             qtlib.WarningMsgBox(_('Missing information'), errormsg)
             return
         return self.accept()
 
-    def validateForm(self):
+    def validateForm(self) -> str:
         return '' # No error
 
-    def _readsettings(self):
+    def _readsettings(self) -> QSettings:
         s = QSettings()
         if self.dialogname:
             self.restoreGeometry(
                 qtlib.readByteArray(s, self.dialogname + '/geom'))
         return s
 
-    def _writesettings(self):
+    def _writesettings(self) -> None:
         s = QSettings()
         if self.dialogname:
             s.setValue(self.dialogname + '/geom', self.saveGeometry())
 
-    def done(self, r):
+    def done(self, r: int) -> None:
         self._writesettings()
         super(CustomConfigDialog, self).done(r)
 
@@ -734,7 +784,7 @@ class CustomConfigDialog(QDialog):
 class CustomToolConfigDialog(CustomConfigDialog):
     '''Dialog for editing custom tool configurations'''
 
-    _enablemappings = [(_('All items'), 'istrue'),
+    _enablemappings: List[Tuple[str, str]] = [(_('All items'), 'istrue'),
                        (_('Working directory'), 'iswd'),
                        (_('All revisions'), 'isrev'),
                        (_('All contexts'), 'isctx'),
@@ -742,9 +792,14 @@ class CustomToolConfigDialog(CustomConfigDialog):
                        (_('Applied patches'), 'applied'),
                        (_('Applied patches or qparent'), 'qgoto'),
                        ]
-    _defaulticonstring = _('<default icon>')
+    _defaulticonstring: str = _('<default icon>')
 
-    def __init__(self, parent=None, toolname=None, toolconfig=None):
+    def __init__(
+        self,
+        parent: Optional[QWidget] = None,
+        toolname: Optional[str] = None,
+        toolconfig: Optional[Dict[str, Union[str, bool]]] = None,
+    ) -> None:
         super(CustomToolConfigDialog, self).__init__(parent,
             dialogname='customtools',
             windowTitle=_('Configure Custom Tool'),
@@ -762,10 +817,10 @@ class CustomToolConfigDialog(CustomConfigDialog):
         enable = toolconfig.get('enable', 'all')
         showoutput = str(toolconfig.get('showoutput', False))
 
-        self.name = self._addConfigItem(vbox, _('Tool name'),
+        self.name: QLineEdit = self._addConfigItem(vbox, _('Tool name'),
             QLineEdit(toolname), _('The tool name. It cannot contain spaces.'))
             # Execute a mercurial command. These _MUST_ start with "hg"
-        self.command = self._addConfigItem(vbox, _('Command'),
+        self.command: QLineEdit = self._addConfigItem(vbox, _('Command'),
             QLineEdit(command), _('The command that will be executed.\n'
             'To execute a Mercurial command use "hg" (rather than "hg.exe") '
             'as the executable command.\n'
@@ -785,19 +840,19 @@ class CustomToolConfigDialog(CustomConfigDialog):
             'hexadecimal revision id hash respectively.\n'
             '- {REV_B} / {REVID_B}: the second selected revision number / '
             'hexadecimal revision id hash respectively.\n'))
-        self.workingdir = self._addConfigItem(vbox, _('Working Directory'),
+        self.workingdir: QLineEdit = self._addConfigItem(vbox, _('Working Directory'),
             QLineEdit(workingdir),
             _('The directory where the command will be executed.\n'
             'If this is not set, the root of the current repository '
             'will be used instead.\n'
             'You can use the same {VARIABLES} as on the "Command" setting.\n'))
-        self.label = self._addConfigItem(vbox, _('Tool label'),
+        self.label: QLineEdit = self._addConfigItem(vbox, _('Tool label'),
             QLineEdit(label),
             _('The tool label, which is what will be shown '
             'on the repowidget context menu.\n'
             'If no label is set, the tool name will be used as the tool label.\n'
             'If no tooltip is set, the label will be used as the tooltip as well.'))
-        self.tooltip = self._addConfigItem(vbox, _('Tooltip'),
+        self.tooltip: QLineEdit = self._addConfigItem(vbox, _('Tooltip'),
             QLineEdit(tooltip),
             _('The tooltip that will be shown on the tool button.\n'
             'This is only shown when the tool button is shown on\n'
@@ -819,7 +874,7 @@ class CustomToolConfigDialog(CustomConfigDialog):
         # it will have been added as the first element on the combobox!
         combo.setCurrentIndex(idx)
 
-        self.icon = self._addConfigItem(vbox, _('Icon'),
+        self.icon: QComboBox = self._addConfigItem(vbox, _('Icon'),
             combo,
             _('The tool icon.\n'
             'You can use any built-in TortoiseHg icon\n'
@@ -830,35 +885,35 @@ class CustomToolConfigDialog(CustomConfigDialog):
 
         combo = self._genCombo([l for l, _v in self._enablemappings],
                                self._enable2label(enable))
-        self.enable = self._addConfigItem(vbox, _('On repowidget, show for'),
+        self.enable: QComboBox = self._addConfigItem(vbox, _('On repowidget, show for'),
             combo,  _('For which kinds of revisions the tool will be enabled\n'
             'It is only taken into account when the tool is shown on the\n'
             'selected revision context menu.'))
 
         combo = self._genCombo(('True', 'False'), showoutput)
-        self.showoutput = self._addConfigItem(vbox, _('Show Output Log'),
+        self.showoutput: QComboBox = self._addConfigItem(vbox, _('Show Output Log'),
             combo, _('When enabled, automatically show the Output Log when the '
             'command is run.\nDefault: False.'))
 
-    def value(self):
-        toolname = str(self.name.text()).strip()
+    def value(self) -> Tuple[str, Dict[str, str]]:  # pytype: disable=signature-mismatch
+        toolname = self.name.text().strip()
         toolconfig = {
-            'label': str(self.label.text()),
-            'command': str(self.command.text()),
-            'workingdir': str(self.workingdir.text()),
-            'tooltip': str(self.tooltip.text()),
-            'icon': str(self.icon.currentText()),
+            'label': self.label.text(),
+            'command': self.command.text(),
+            'workingdir': self.workingdir.text(),
+            'tooltip': self.tooltip.text(),
+            'icon': self.icon.currentText(),
             'enable': self._enablemappings[self.enable.currentIndex()][1],
-            'showoutput': str(self.showoutput.currentText()),
+            'showoutput': self.showoutput.currentText(),
         }
         if toolconfig['icon'] == self._defaulticonstring:
             toolconfig['icon'] = ''
         return toolname, toolconfig
 
-    def _enable2label(self, value):
+    def _enable2label(self, value: str) -> Optional[str]:
         return dict((v, l) for l, v in self._enablemappings).get(value)
 
-    def validateForm(self):
+    def validateForm(self) -> str:
         name, config = self.value()
         if not name:
             return _('You must set a tool name.')
@@ -975,9 +1030,15 @@ class HookConfigDialog(CustomConfigDialog):
             'conflicts not resolved), <tt>$HG_ERROR=1</tt>.'),
     )
 
-    _rehookname = re.compile('^[^=\s]*$')
+    _rehookname: Pattern[str] = re.compile('^[^=\s]*$')
 
-    def __init__(self, parent=None, hooktype=None, command='', hookname=''):
+    def __init__(
+        self,
+        parent: Optional[QWidget] = None,
+        hooktype: Optional[str] = None,
+        command: str = '',
+        hookname: str = '',
+    ) -> None:
         super(HookConfigDialog, self).__init__(parent,
             dialogname='hookconfigdialog',
             windowTitle=_('Configure Hook'),
@@ -985,22 +1046,22 @@ class HookConfigDialog(CustomConfigDialog):
 
         vbox = self.formvbox
         combo = self._genCombo(self._hooktypes, hooktype, self._hooktooltips)
-        self.hooktype = self._addConfigItem(vbox, _('Hook type'),
+        self.hooktype: QComboBox = self._addConfigItem(vbox, _('Hook type'),
             combo, _('Select when your command will be run'))
-        self.name = self._addConfigItem(vbox, _('Tool name'),
+        self.name: QLineEdit = self._addConfigItem(vbox, _('Tool name'),
             QLineEdit(hookname), _('The hook name. It cannot contain spaces.'))
-        self.command = self._addConfigItem(vbox, _('Command'),
+        self.command: QLineEdit = self._addConfigItem(vbox, _('Command'),
             QLineEdit(command), _('The command that will be executed.\n'
                  'To execute a python function prepend the command with '
                  '"python:".\n'))
 
-    def value(self):
-        hooktype = str(self.hooktype.currentText())
-        hookname = str(self.name.text()).strip()
-        command = str(self.command.text()).strip()
+    def value(self) -> Tuple[str, str, str]:  # pytype: disable=signature-mismatch
+        hooktype = self.hooktype.currentText()
+        hookname = self.name.text().strip()
+        command = self.command.text().strip()
         return hooktype, command, hookname
 
-    def validateForm(self):
+    def validateForm(self) -> str:
         hooktype, command, hookname = self.value()
         if hooktype not in self._hooktypes:
             return _('You must set a valid hook type.')
