@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import os
 
@@ -182,7 +182,7 @@ class _LogWidgetForConsole(cmdui.LogWidget):
 
     def _ensurePrompt(self, line):
         """Insert prompt string if not available"""
-        s = pycompat.unicode(self.text(line))
+        s = self.text(line)
         if s.startswith(self._prompt):
             return
         for i, c in enumerate(self._prompt):
@@ -196,7 +196,7 @@ class _LogWidgetForConsole(cmdui.LogWidget):
             return self._savedcommands[-1]
         l = self._findPromptLine()
         if l >= 0:
-            return pycompat.unicode(self.text(l))[len(self._prompt):].rstrip('\n')
+            return self.text(l)[len(self._prompt):].rstrip('\n')
         else:
             return ''
 
@@ -313,9 +313,9 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
             setattr(self, name, getattr(self._logwidget, name))
 
     @pyqtSlot(str, int)
-    def historySearch(self, text, direction):
+    def historySearch(self, text: str, direction: int):
         cmdline, idx = _searchhistory(self._commandHistory,
-                                      pycompat.unicode(text),
+                                      text,
                                       direction, self._commandIdx)
         if cmdline:
             self._commandIdx = idx
@@ -374,12 +374,12 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
         return sorted(matches)
 
     @pyqtSlot(str)
-    def completeCommandText(self, text):
+    def completeCommandText(self, text: str):
         """Show the list of history or known commands matching the search text
 
         Also complete the prompt with the common prefix to the matching items
         """
-        text = pycompat.unicode(text).strip()
+        text = text.strip()
         if not text:
             self._logwidget.flash()
             return
@@ -470,14 +470,13 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
             self.openPrompt()
 
     @pyqtSlot(str)
-    def _runcommand(self, cmdline):
-        cmdline = pycompat.unicode(cmdline)
+    def _runcommand(self, cmdline: str):
         self._commandIdx = 0
         try:
             args = hglib.parsecmdline(cmdline, self._workingDirectory())
         except ValueError as e:
             self.closePrompt()
-            self._logwidget.appendLog(pycompat.unicode(e) + '\n', 'ui.error')
+            self._logwidget.appendLog(hglib.exception_str(e) + '\n', 'ui.error')
             self.openPrompt()
             return
         if not args:
@@ -570,15 +569,13 @@ class LogDockWidget(QDockWidget):
         return w
 
     @pyqtSlot(str)
-    def _createConsoleFor(self, root):
-        root = pycompat.unicode(root)
+    def _createConsoleFor(self, root: str):
         repoagent = self._repomanager.repoAgent(root)
         assert repoagent
         self._createConsole(repoagent)
 
     @pyqtSlot(str)
-    def _destroyConsoleFor(self, root):
-        root = pycompat.unicode(root)
+    def _destroyConsoleFor(self, root: str):
         w = self._findConsoleFor(root)
         self._consoles.removeWidget(w)
         w.setParent(None)
