@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import difflib
 import os
@@ -13,11 +13,11 @@ import re
 
 from . import qsci as Qsci
 from .qtcore import (
+    QElapsedTimer,
     QEvent,
     QObject,
     QPoint,
     QSettings,
-    QTime,
     QTimer,
     Qt,
     pyqtSignal,
@@ -301,7 +301,7 @@ class HgFileView(QFrame):
         # TODO: pass self._fd.canonicalFilePath()
         dlg = shelve.ShelveDialog(self._repoagent, self)
         dlg.finished.connect(dlg.deleteLater)
-        dlg.exec_()
+        dlg.exec()
         self.shelveToolExited.emit()
 
     def setShelveButtonVisible(self, visible):
@@ -418,10 +418,10 @@ class HgFileView(QFrame):
             self._lostMode = mode
 
     @pyqtSlot(QAction)
-    def _setParentRevision(self, action):
+    def _setParentRevision(self, action: QAction) -> None:
         fd = self._fd
         ctx = fd.rawContext()
-        pctx = {'1': ctx.p1, '2': ctx.p2}[str(action.text())]()
+        pctx = {'1': ctx.p1, '2': ctx.p2}[action.text()]()
         self.display(fd.createRebased(pctx))
 
     def _updateFileDataActions(self):
@@ -641,7 +641,7 @@ class HgFileView(QFrame):
         path = hglib.fromunicode(path)
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [path], opts)
         if dlg:
-            dlg.exec_()
+            dlg.exec()
 
     @pyqtSlot(str, int)
     def _visualDiffRevision(self, path, rev):
@@ -654,7 +654,7 @@ class HgFileView(QFrame):
     @pyqtSlot(QPoint)
     def _onMenuRequested(self, point):
         menu = self._createContextMenu(point)
-        menu.exec_(self.sci.viewport().mapToGlobal(point))
+        menu.exec(self.sci.viewport().mapToGlobal(point))
         menu.setParent(None)
 
     def _createContextMenu(self, point):
@@ -690,7 +690,7 @@ class HgFileView(QFrame):
             font = self.sci.font()
         fm = QFontMetrics(font)
 
-        lines = pycompat.unicode(self.sci.text()).splitlines()
+        lines = self.sci.text().splitlines()
         if lines:
             # assume that the longest line has the largest width;
             # fm.width() is too slow to apply to each line.
@@ -759,7 +759,7 @@ class _DiffViewControl(_AbstractViewControl):
         self._startBuildMarker()
 
     def _startBuildMarker(self):
-        self._linestoprocess = pycompat.unicode(self._sci.text()).splitlines()
+        self._linestoprocess = self._sci.text().splitlines()
         self._firstlinetoprocess = 0
         self._buildtimer.start()
 
@@ -986,8 +986,7 @@ class _AnnotateViewControl(_AbstractViewControl):
         self._revmarkers = {}  # by rev
         self._lastrev = -1
 
-        self._lastmarginclick = QTime.currentTime()
-        self._lastmarginclick.addMSecs(-QApplication.doubleClickInterval())
+        self._lastmarginclick = QElapsedTimer()
 
         self._initAnnotateOptionActions()
         self._loadAnnotateSettings()
@@ -1152,7 +1151,7 @@ class _AnnotateViewControl(_AbstractViewControl):
                                 s.style(), s.paper())
         self._sci.SendScintilla(qsci.SCI_STYLESETFONT,
                                 s.style(),
-                                pycompat.unicode(s.font().family()).encode('utf-8'))
+                                s.font().family().encode('utf-8'))
         self._sci.SendScintilla(qsci.SCI_STYLESETSIZE,
                                 s.style(), s.font().pointSize())
         for i, (fctx, _origline) in enumerate(self._links):
@@ -1263,14 +1262,15 @@ class _AnnotateViewControl(_AbstractViewControl):
                     action.triggered.connect(action.run)
                 add(name, func)
 
-    #@pyqtSlot(int, int, Qt.KeyboardModifiers)
+    #@pyqtSlot(int, int, Qt.KeyboardModifier)
     def _onMarginClicked(self, margin, line, state):
         if margin != _AnnotateMargin:
             return
 
+        interval = QApplication.doubleClickInterval()
         lastclick = self._lastmarginclick
         if (state == Qt.KeyboardModifier.ControlModifier
-            or lastclick.elapsed() < QApplication.doubleClickInterval()):
+            or (lastclick.isValid() and not lastclick.hasExpired(interval))):
             if line >= len(self._links):
                 # empty line next to the last line
                 return
