@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import re
 
@@ -87,7 +87,7 @@ class MessageEntry(qscilib.Scintilla):
     @pyqtSlot(QPoint)
     def menuRequested(self, point):
         menu = self._createContextMenu(point)
-        menu.exec_(self.viewport().mapToGlobal(point))
+        menu.exec(self.viewport().mapToGlobal(point))
         menu.setParent(None)
 
     def _createContextMenu(self, point):
@@ -113,7 +113,7 @@ class MessageEntry(qscilib.Scintilla):
         def settings():
             from tortoisehg.hgqt.settings import SettingsDialog
             dlg = SettingsDialog(True, focus='tortoisehg.summarylen')
-            dlg.exec_()
+            dlg.exec()
         def togglelexer():
             QSettings().setValue('msgentry/lexer', not lexerenabled)
             self.applylexer()
@@ -143,7 +143,7 @@ class MessageEntry(qscilib.Scintilla):
         self.summarylen = repo.summarylen
 
     def reflowBlock(self, line):
-        lines = pycompat.unicode(self.text()).splitlines()
+        lines = self.text().splitlines()
         if line >= len(lines):
             return None
         if not len(lines[line]) > 1:
