@@ -5,10 +5,15 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import os
 
+from typing import (
+    cast,
+    Optional,
+)
+
 from .qtcore import (
     QFile,
     QFileSystemWatcher,
@@ -55,7 +60,7 @@ from . import (
 def settingsfilename():
     """Return path to thg-reporegistry.xml as unicode"""
     s = QSettings()
-    dir = os.path.dirname(pycompat.unicode(s.fileName()))
+    dir = os.path.dirname(s.fileName())
     return dir + '/' + 'thg-reporegistry.xml'
 
 
@@ -158,7 +163,7 @@ class RepoTreeView(QTreeView):
                 # Event is a drop of an external repo
                 accept = False
                 for u in data.urls():
-                    uroot = paths.find_root(pycompat.unicode(u.toLocalFile()))
+                    uroot = paths.find_root(u.toLocalFile())
                     if uroot and not m.isKnownRepoRoot(uroot, standalone=True):
                         repoindex = m.addRepo(uroot, row, group)
                         m.loadSubrepos(repoindex)
@@ -411,9 +416,8 @@ class RepoRegistryView(QDockWidget):
 
     # TODO: better to handle repositoryOpened signal by model
     @pyqtSlot(str)
-    def _addAndScanRepo(self, uroot):
+    def _addAndScanRepo(self, uroot: str):
         """Add repo if not exists; called when the workbench has opened it"""
-        uroot = pycompat.unicode(uroot)
         m = self.tview.model()
         assert m is not None
         knownindex = m.indexFromRepoRoot(uroot)
@@ -546,7 +550,7 @@ class RepoRegistryView(QDockWidget):
     ## Menu action handlers
     #
 
-    def _currentRepoRoot(self):
+    def _currentRepoRoot(self) -> Optional[str]:
         model = self.tview.model()
         index = self.tview.currentIndex()
         assert model is not None
@@ -583,7 +587,7 @@ class RepoRegistryView(QDockWidget):
         if path:
             m = self.tview.model()
             assert m is not None
-            uroot = paths.find_root(pycompat.unicode(path))
+            uroot = paths.find_root(path)
             if uroot and not m.isKnownRepoRoot(uroot, standalone=True):
                 index = m.addRepo(uroot, parent=self.tview.currentIndex())
                 self._scanAddedRepo(index)
@@ -591,10 +595,12 @@ class RepoRegistryView(QDockWidget):
     def addSubrepo(self):
         'menu action handler for adding a new subrepository'
         root = self._currentRepoRoot()
+        assert root is not None
+
         caption = _('Select an existing repository to add as a subrepo')
         FD = QFileDialog
-        path = pycompat.unicode(FD.getExistingDirectory(caption=caption,
-            directory=root, options=QFileDialog.Option.ShowDirsOnly | QFileDialog.Option.ReadOnly))
+        path = FD.getExistingDirectory(caption=caption,
+            directory=root, options=QFileDialog.Option.ShowDirsOnly | QFileDialog.Option.ReadOnly)
         if path:
             path = os.path.normpath(path)
             sroot = paths.find_root(path)
@@ -629,8 +635,7 @@ class RepoRegistryView(QDockWidget):
                 # the selected repository
 
                 # Use forward slashes for relative subrepo root paths
-                srelroot = sroot[len(root)+1:]
-                srelroot = util.pconvert(srelroot)
+                srelroot = util.pconvert(hglib.fromunicode(sroot[len(root)+1:]))
 
                 # Is is already on the selected repository substate list?
                 try:
@@ -642,11 +647,12 @@ class RepoRegistryView(QDockWidget):
                         'cannot be open!') % root, parent=self)
                     return
 
-                if hglib.fromunicode(srelroot) in repo[b'.'].substate:
+                if srelroot in repo[b'.'].substate:
                     qtlib.WarningMsgBox(_('Subrepository already exists'),
                         _('The selected repository:<br><br>%s<br><br>'
                         'is already a subrepository of:<br><br>%s<br><br>'
-                        'as: "%s"') % (sroot, root, srelroot), parent=self)
+                        'as: "%s"') % (sroot, root, hglib.tounicode(srelroot)),
+                                        parent=self)
                     return
                 else:
                     # Read the current .hgsub file contents
@@ -665,29 +671,28 @@ class RepoRegistryView(QDockWidget):
 
                     # Make sure that the selected subrepo (or one of its
                     # subrepos!) is not already on the .hgsub file
-                    linesep = ''
+                    linesep = b''
                     # On Windows case is unimportant, while on posix it is
                     srelrootnormcase = os.path.normcase(srelroot)
                     for line in lines:
-                        line = hglib.tounicode(line)
-                        spath = line.split("=")[0].strip()
+                        spath = line.split(b"=")[0].strip()
                         if not spath:
                             continue
+                        uline = hglib.tounicode(line)
                         if not linesep:
-                            linesep = hglib.getLineSeparator(line)
+                            linesep = hglib.fromunicode(hglib.getLineSeparator(uline))
                         spath = util.pconvert(spath)
                         if os.path.normcase(spath) == srelrootnormcase:
                             qtlib.WarningMsgBox(
                                 _('Failed to add repository'),
                                 _('The .hgsub file already contains the '
-                                'line:<br><br>%s') % line, parent=self)
+                                'line:<br><br>%s') % uline, parent=self)
                             return
                     if not linesep:
-                        linesep = os.linesep
+                        linesep = pycompat.oslinesep
 
                     # Append the new subrepo to the end of the .hgsub file
-                    lines.append(hglib.fromunicode('%s = %s'
-                                                   % (srelroot, srelroot)))
+                    lines.append(b'%s = %s' % (srelroot, srelroot))
                     lines = [line.strip(linesep) for line in lines]
 
                     # and update the .hgsub file
@@ -706,7 +711,7 @@ class RepoRegistryView(QDockWidget):
                             'subrepo <i>you must still <u>commit</u></i> the '
                             'changes to the .hgsub file in order to confirm '
                             'the addition of the subrepo.') \
-                            % (srelroot, root), parent=self)
+                            % (hglib.tounicode(srelroot), root), parent=self)
                     except:
                         qtlib.WarningMsgBox(
                             _('Failed to add repository'),
@@ -716,22 +721,23 @@ class RepoRegistryView(QDockWidget):
 
     def removeSubrepo(self):
         'menu action handler for removing an existing subrepository'
-        model = self.tview.model()
+        model = cast(repotreemodel.RepoTreeModel, self.tview.model())
         index = self.tview.currentIndex()
         assert model is not None
-        path = model.repoRoot(index)
-        root = model.repoRoot(index.parent())
-        relsubpath = os.path.normcase(os.path.normpath(path[1+len(root):]))
-        hgsubfilename = os.path.join(root, '.hgsub')
+        upath = model.repoRoot(index)
+        uroot = model.repoRoot(index.parent())
+        urelsubpath = os.path.normcase(os.path.normpath(upath[1+len(uroot):]))
+        relsubpath = hglib.fromunicode(urelsubpath)
+        hgsubfilename = os.path.join(hglib.fromunicode(uroot), b'.hgsub')
 
         try:
             hgsub = []
             found = False
-            with open(hgsubfilename, 'r') as f:
+            with open(hgsubfilename, 'rb') as f:
                 for line in f:
                     spath = os.path.normcase(
                         os.path.normpath(
-                            line.split('=')[0].strip()))
+                            line.split(b'=')[0].strip()))
                     if spath != relsubpath:
                         hgsub.append(line)
                     else:
@@ -753,12 +759,12 @@ class RepoRegistryView(QDockWidget):
         choices = (_('&Yes'), _('&No'))
         answer = qtlib.CustomPrompt(_('Remove the selected repository?'),
             _('Do you really want to remove the repository "<i>%s</i>" '
-              'from its parent repository "<i>%s</i>"') % (relsubpath, root),
-            self, choices=choices, default=choices[0]).run()
+              'from its parent repository "<i>%s</i>"') % (urelsubpath, uroot),
+            self, choices=choices, default=0).run()
         if answer != 0:
             return
         try:
-            with open(hgsubfilename, 'w') as f:
+            with open(hgsubfilename, 'wb') as f:
                 f.writelines(hgsub)
             qtlib.InfoMsgBox(_('Subrepository removed from .hgsub'),
                 _('The selected subrepository has been removed '
@@ -777,7 +783,7 @@ class RepoRegistryView(QDockWidget):
         sd = settings.SettingsDialog(configrepo=True, focus='web.name',
                                      parent=self, root=root)
         sd.finished.connect(sd.deleteLater)
-        sd.exec_()
+        sd.exec()
 
     def openAll(self):
         index = self.tview.currentIndex()
@@ -924,8 +930,7 @@ class RepoRegistryView(QDockWidget):
                 % (root, "<br>".join(invalidpaths)), parent=self)
 
     @pyqtSlot(str)
-    def scanRepo(self, uroot):
-        uroot = pycompat.unicode(uroot)
+    def scanRepo(self, uroot: str):
         m = self.tview.model()
         assert m is not None
         index = m.indexFromRepoRoot(uroot)
