@@ -6,13 +6,15 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import os
 
 from .qtcore import (
+    QObject,
     QPoint,
     QSignalMapper,
+    QT_API,
     Qt,
     pyqtSignal,
     pyqtSlot,
@@ -113,9 +115,17 @@ class RepoTabWidget(QWidget):
         self._lastclosedpaths = []
 
         self._iconmapper = QSignalMapper(self)
-        self._iconmapper.mapped[QWidget].connect(self._updateIcon)
         self._titlemapper = QSignalMapper(self)
-        self._titlemapper.mapped[QWidget].connect(self._updateTitle)
+
+        if QT_API == 'PyQt5':
+            iconSignal = self._iconmapper.mapped[QObject]
+            titleSignal = self._titlemapper.mapped[QObject]
+        else:
+            iconSignal = self._iconmapper.mappedObject[QObject]
+            titleSignal = self._titlemapper.mappedObject[QObject]
+
+        iconSignal.connect(self._updateIcon)
+        titleSignal.connect(self._updateTitle)
 
         self._updateTabSwitchActions()
 
@@ -326,7 +336,7 @@ class RepoTabWidget(QWidget):
         return self.repoRootPath(self.currentIndex())
 
     def repoRootPath(self, index):
-        return pycompat.unicode(self._tabbar.tabToolTip(index))
+        return self._tabbar.tabToolTip(index)
 
     def _findIndexesByRepoRootPath(self, root):
         for i in pycompat.xrange(self.count()):
@@ -414,24 +424,22 @@ class RepoTabWidget(QWidget):
         rw.taskTabVisibilityChanged.connect(self.taskTabVisibilityChanged)
         rw.toolbarVisibilityChanged.connect(self.toolbarVisibilityChanged)
         rw.busyIconChanged.connect(self._iconmapper.map)
-        self._iconmapper.setMapping(rw, rw)
+        self._iconmapper.setMapping(rw, qtlib.sip.cast(rw, QObject))
         rw.titleChanged.connect(self._titlemapper.map)
-        self._titlemapper.setMapping(rw, rw)
+        self._titlemapper.setMapping(rw, qtlib.sip.cast(rw, QObject))
         self._stack.addWidget(rw)
         return rw
 
     @pyqtSlot(str, object, str, str, object)
-    def _mapProgressReceived(self, topic, pos, item, unit, total):
+    def _mapProgressReceived(self, topic: str, pos, item: str, unit: str, total):
         rw = self.sender()
         assert isinstance(rw, repowidget.RepoWidget), repr(rw)
-        progress = cmdcore.ProgressMessage(
-            pycompat.unicode(topic), pos, pycompat.unicode(item),
-            pycompat.unicode(unit), total)
+        progress = cmdcore.ProgressMessage(topic, pos, item, unit, total)
         self.progressReceived.emit(rw.repoRootPath(), progress)
 
     @pyqtSlot(str)
-    def _openLinkedRepo(self, path):
-        uri = pycompat.unicode(path).split('?', 1)
+    def _openLinkedRepo(self, path: str):
+        uri = path.split('?', 1)
         path = hglib.normreporoot(uri[0])
         rev = None
         if len(uri) > 1:
@@ -445,12 +453,12 @@ class RepoTabWidget(QWidget):
                 # case, the user is going to commit changes to this repo.
                 rw.switchToNamedTaskTab('commit')
 
-    @pyqtSlot('QWidget*')
+    @pyqtSlot(QObject)  # really QWidget
     def _updateIcon(self, rw):
         index = self._indexOf(rw)
         self._tabbar.setTabIcon(index, rw.busyIcon())
 
-    @pyqtSlot('QWidget*')
+    @pyqtSlot(QObject)  # really QWidget
     def _updateTitle(self, rw):
         index = self._indexOf(rw)
         self._tabbar.setTabText(index, rw.title())
