@@ -5,9 +5,12 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import, print_function
+
 
 import os
+from typing import (
+    Optional,
+)
 
 from .qtcore import (
     QEvent,
@@ -23,8 +26,8 @@ from .qtgui import (
     QCompleter,
     QDialog,
     QDialogButtonBox,
-    QDirModel,
     QFileDialog,
+    QFileSystemModel,
     QFont,
     QFontDialog,
     QFormLayout,
@@ -126,7 +129,7 @@ class SettingsCombo(QComboBox):
         else:
             settings = opts['settings']
             slist = qtlib.readStringList(settings, 'settings/' + opts['cpath'])
-            self.previous = [pycompat.unicode(s) for s in slist if s]
+            self.previous = [s for s in slist if s]
         self.setMinimumWidth(ENTRY_WIDTH)
 
     def resetList(self):
@@ -174,7 +177,7 @@ class SettingsCombo(QComboBox):
         self.resetList()
 
     def value(self):
-        utext = pycompat.unicode(self.currentText())
+        utext = self.currentText()
         if utext == _unspecstr:
             return None
         if ('nohist' in self.opts or utext in self.defaults + self.previous
@@ -210,7 +213,7 @@ class BoolRBGroup(QWidget):
     def setValue(self, curvalue):
         # Store parsed value, so a compare of 1 to True isn't flagged as dirty
         if curvalue:
-            value = pycompat.unicode(stringutil.parsebool(curvalue))
+            value = str(stringutil.parsebool(curvalue))
             self.curvalue = hglib.fromunicode(value)
         else:
             self.curvalue = None
@@ -286,8 +289,7 @@ class TextEntry(QTextEdit):
     def isDirty(self):
         return self.value() != self.curvalue
 
-    def removeEmptyLines(self, text):
-        # type: (pycompat.unicode) -> pycompat.unicode
+    def removeEmptyLines(self, text: str) -> str:
         if not text:
             return text
         rawlines = text.splitlines()
@@ -299,11 +301,11 @@ class TextEntry(QTextEdit):
         return os.linesep.join(lines)
 
 
-def _describeFont(font):
+def _describeFont(font: Optional[QFont]) -> str:
     if not font:
         return _unspecstr
 
-    s = pycompat.unicode(font.family())
+    s = font.family()
     s += ", "  + _("%dpt") % font.pointSize()
     if font.bold():
         s += ", " + _("Bold")
@@ -485,7 +487,9 @@ class PathBrowser(QWidget):
         # use QCompleter(model, parent) to avoid ownership bug of
         # QCompleter(parent /TransferBack/) in PyQt<4.11.4
         completer = QCompleter(None, self)
-        completer.setModel(QDirModel(completer))
+        fsModel = QFileSystemModel(completer)
+        fsModel.setRootPath("")
+        completer.setModel(fsModel)
         self.lineEdit.setCompleter(completer)
 
         self.browseButton = QPushButton(_('&Browse...'))
@@ -1162,7 +1166,7 @@ INFO = (
           'match in the commit message.')),
     _fi(_('Issue Tracker Plugin'), 'tortoisehg.issue.bugtraqplugin',
         (genDeferredCombo, findIssueTrackerPlugins),
-        _('Configures a COM IBugTraqProvider or IBugTrackProvider2 issue '
+        _('Configures a COM IBugTraqProvider or IBugTraqProvider2 issue '
           'tracking plugin.'), visible=issuePluginVisible),
     _fi(_('Configure Issue Tracker'), 'tortoisehg.issue.bugtraqparameters',
         genBugTraqEdit,
@@ -1400,8 +1404,8 @@ class SettingsDialog(QDialog):
                    for i in pycompat.xrange(self.conftabs.count()))
 
     @pyqtSlot(str)
-    def _pushRestartRequest(self, key):
-        self._restartreqs.add(pycompat.unicode(key))
+    def _pushRestartRequest(self, key: str) -> None:
+        self._restartreqs.add(key)
 
     def applyChanges(self):
         results = [self.conftabs.widget(i).applyChanges()
@@ -1540,7 +1544,7 @@ class SettingsForm(QWidget):
     @pyqtSlot(int)
     def activatePage(self, index):
         if index >= 0:
-            self._activepagename = pycompat.unicode(INFO[index][0]['name'])
+            self._activepagename = INFO[index][0]['name']
 
         stackindex = self.pageListIndexToStack.get(index, -1)
         if stackindex >= 0:
@@ -1763,7 +1767,7 @@ class SettingsForm(QWidget):
         return self.ini.get(section, key)
 
     def loadIniFile(self, rcpath):
-        # type: (List[pycompat.unicode]) -> IniConfig
+        # type: (List[str]) -> IniConfig
         for fn in rcpath:
             if os.path.exists(fn):
                 break
@@ -1826,7 +1830,7 @@ class SettingsForm(QWidget):
             return True
         except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
-                                hglib.tounicode(str(e)), parent=self)
+                                hglib.exception_str(e), parent=self)
             return False
 
     def applyChangesForExtensions(self):
