@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import os
 import re
@@ -13,6 +13,13 @@ import stat
 import subprocess
 import threading
 
+from typing import (
+    Dict,
+    Text,
+    Tuple,
+    Union,
+)
+
 from .qtcore import (
     QTimer,
     pyqtSlot,
@@ -50,15 +57,11 @@ from . import qtlib
 if hglib.TYPE_CHECKING:
     from typing import (
         Any,
-        Dict,
         Iterable,
         List,
         Optional,
         Sequence,
         Set,
-        Text,
-        Tuple,
-        Union,
     )
     from mercurial import (
         localrepo,
@@ -198,7 +201,7 @@ def launchtool(cmd, opts, replace, block):
     except (OSError, EnvironmentError) as e:
         QMessageBox.warning(None,
                 _('Tool launch failure'),
-                _('%s : %s') % (hglib.tounicode(cmd), hglib.tounicode(str(e))))
+                _('%s : %s') % (hglib.tounicode(cmd), hglib.exception_str(e)))
 
 def filemerge(ui, fname, patchedfname):
     # type: (uimod.ui, Text, Text) -> None
@@ -211,10 +214,17 @@ def filemerge(ui, fname, patchedfname):
         return None
     preferred = besttool(ui, detectedtools)
     diffcmd, diffopts, mergeopts = detectedtools[preferred]
-    replace = dict(parent=fname, parent1=fname,
-                   plabel1=fname + _('[working copy]'),
-                   repo='', phash1='', phash2='', chash='',
-                   child=patchedfname, clabel=_('[original]'))
+    replace = {
+        "parent": fname,
+        "parent1": fname,
+        "plabel1": fname + _('[working copy]'),
+        "repo": '',
+        "phash1": '',
+        "phash2": '',
+        "chash": '',
+        "child": patchedfname,
+        "clabel": _('[original]'),
+    }
     launchtool(diffcmd, diffopts, replace, True)
 
 
@@ -401,8 +411,9 @@ def visual_diff(ui, repo, pats, ctx1a, ctx1b, ctx2, tool, mainapp=False,
         if len(MAR) > 1 and label2 == b'':
             label2 = b'working files'
 
-        def getfile(fname, dir, label):
-            # type: (bytes, bytes, bytes) -> Tuple[bytes, bytes]
+        def getfile(
+            fname: bytes, dir: bytes, label: bytes
+        ) -> Tuple[bytes, bytes]:
             file = os.path.join(qtlib.gettempdir(), dir, fname)
             if os.path.isfile(file):
                 return fname+label, file
@@ -433,11 +444,19 @@ def visual_diff(ui, repo, pats, ctx1a, ctx1b, ctx2, tool, mainapp=False,
         repoagent = repo._pyqtobj  # TODO
 
         # TODO: sort out bytes vs str
-        replace = dict(parent=dir1a, parent1=dir1a, parent2=dir1b,
-                       plabel1=label1a, plabel2=label1b,
-                       phash1=str(ctx1a), phash2=str(ctx1b),
-                       repo=hglib.fromunicode(repoagent.displayName()),
-                       clabel=label2, child=dir2, chash=str(ctx2))  # type: Dict[Text, Union[bytes, Text]]
+        replace: Dict[Text, Union[bytes, Text]] = {
+            "parent": dir1a,
+            "parent1": dir1a,
+            "parent2": dir1b,
+            "plabel1": label1a,
+            "plabel2": label1b,
+            "phash1": str(ctx1a),
+            "phash2": str(ctx1b),
+            "repo": hglib.fromunicode(repoagent.displayName()),
+            "clabel": label2,
+            "child": dir2,
+            "chash": str(ctx2),
+        }
         launchtool(diffcmd, args, replace, True)
 
         # detect if changes were made to mirrored working files
@@ -524,7 +543,7 @@ class FileSelectionDialog(QDialog):
             combo.setCurrentIndex(defrow)
 
             list.currentRowChanged.connect(self.updateToolSelection)
-            combo.currentIndexChanged[str].connect(self.onToolSelected)
+            combo.currentTextChanged.connect(self.onToolSelected)
             self.toolCombo = combo
 
         bb = QDialogButtonBox()
@@ -663,7 +682,7 @@ class FileSelectionDialog(QDialog):
         if ctx1b:
             other, file1b = getfile(ctx1b, dir1b, fname, source)
         else:
-            other, file1b = fname, None
+            other, file1b = fname, b''
         fname, file2 = getfile(ctx2, dir2, fname, None)  # pytype: disable=annotation-type-mismatch
 
         label1a = local+rev1a
@@ -675,11 +694,19 @@ class FileSelectionDialog(QDialog):
             label2 += b'[merged]'
 
         # Function to quote file/dir names in the argument string
-        replace = dict(parent=file1a, parent1=file1a, plabel1=label1a,
-                       parent2=file1b, plabel2=label1b,
-                       repo=self.reponame,
-                       phash1=str(ctx1a), phash2=str(ctx1b), chash=str(ctx2),
-                       clabel=label2, child=file2)  # type: Dict[Text, Union[bytes, Text]]
+        replace: Dict[Text, Union[bytes, Text]] = {
+            "parent": file1a,
+            "parent1": file1a,
+            "plabel1": label1a,
+            "parent2": file1b,
+            "plabel2": label1b,
+            "repo": self.reponame,
+            "phash1": str(ctx1a),
+            "phash2": str(ctx1b),
+            "chash": str(ctx2),
+            "clabel": label2,
+            "child": file2,
+        }
         args = ctx1b and self.mergeopts or self.diffopts
         launchtool(self.diffpath, args, replace, False)
 
@@ -689,10 +716,19 @@ class FileSelectionDialog(QDialog):
         rev1a, rev1b, rev2 = self.revs
         ctx1a, ctx1b, ctx2 = self.ctxs
 
-        replace = dict(parent=dir1a, parent1=dir1a, plabel1=rev1a,
-                       repo=self.reponame,
-                       phash1=str(ctx1a), phash2=str(ctx1b), chash=str(ctx2),
-                       parent2='', plabel2='', clabel=rev2, child=dir2)  # type: Dict[Text, Union[bytes, Text]]
+        replace: Dict[Text, Union[bytes, Text]] = {
+            "parent": dir1a,
+            "parent1": dir1a,
+            "plabel1": rev1a,
+            "repo": self.reponame,
+            "phash1": str(ctx1a),
+            "phash2": str(ctx1b),
+            "chash": str(ctx2),
+            "parent2": '',
+            "plabel2": '',
+            "clabel": rev2,
+            "child": dir2,
+        }
         launchtool(self.diffpath, self.diffopts, replace, False)
 
     def p2dirdiff(self):
@@ -701,10 +737,19 @@ class FileSelectionDialog(QDialog):
         rev1a, rev1b, rev2 = self.revs
         ctx1a, ctx1b, ctx2 = self.ctxs
 
-        replace = dict(parent=dir1b, parent1=dir1b, plabel1=rev1b,
-                       repo=self.reponame,
-                       phash1=str(ctx1a), phash2=str(ctx1b), chash=str(ctx2),
-                       parent2='', plabel2='', clabel=rev2, child=dir2)  # type: Dict[Text, Union[bytes, Text]]
+        replace: Dict[Text, Union[bytes, Text]] = {
+            "parent": dir1b,
+            "parent1": dir1b,
+            "plabel1": rev1b,
+            "repo": self.reponame,
+            "phash1": str(ctx1a),
+            "phash2": str(ctx1b),
+            "chash": str(ctx2),
+            "parent2": '',
+            "plabel2": '',
+            "clabel": rev2,
+            "child": dir2,
+        }
         launchtool(self.diffpath, self.diffopts, replace, False)
 
     def threewaydirdiff(self):
@@ -713,8 +758,17 @@ class FileSelectionDialog(QDialog):
         rev1a, rev1b, rev2 = self.revs
         ctx1a, ctx1b, ctx2 = self.ctxs
 
-        replace = dict(parent=dir1a, parent1=dir1a, plabel1=rev1a,
-                       repo=self.reponame,
-                       phash1=str(ctx1a), phash2=str(ctx1b), chash=str(ctx2),
-                       parent2=dir1b, plabel2=rev1b, clabel=dir2, child=rev2)  # type: Dict[Text, Union[bytes, Text]]
+        replace: Dict[Text, Union[bytes, Text]] = {
+            "parent": dir1a,
+            "parent1": dir1a,
+            "plabel1": rev1a,
+            "repo": self.reponame,
+            "phash1": str(ctx1a),
+            "phash2": str(ctx1b),
+            "chash": str(ctx2),
+            "parent2": dir1b,
+            "plabel2": rev1b,
+            "clabel": dir2,
+            "child": rev2,
+        }
         launchtool(self.diffpath, self.mergeopts, replace, False)
